<?php

App::uses('BuzzVouchersAppController', 'BuzzVouchers.Controller');

class GiftVoucherPurchasesController extends BuzzVouchersAppController {

	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'index',
			'vouchers'
		));
	}

/**
 * Gift vouchers landing page - will mostly get bypassed by homepage/activity pages
 *
 * @return void
 */
	public function index() {
		if (!empty($this->request->data)) {

			if (!empty($this->request->data['GiftVoucherPurchase']['voucher_activity_id'])) {
				// Forward to the correct vouchers step.
				return $this->redirect([
					'action' => 'vouchers',
					$this->request->data['GiftVoucherPurchase']['voucher_activity_id']
				]);
			} else {
				$this->Session->setFlash(
					__d('BuzzVouchers', 'Please select an activity'),
					'flash_fail'
				);
			}

		}

		$this->loadModel('BuzzVouchers.VoucherActivity');
		$voucherActivities = $this->VoucherActivity->getListed();
		$voucherActivities = Hash::combine($voucherActivities, '{n}.VoucherActivity.id', '{n}.VoucherActivity.name');

		if (count($voucherActivities) === 1) {
			return $this->redirect([
				'action' => 'vouchers'
			]);
		} elseif (empty($voucherActivities)) {
			// No activities have been setup or created so throw an exception
			throw new NotFoundException();
		}

		$this->set(compact('voucherActivities'));
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/index';

		return;
	}

/**
 * Vouchers step - add gift vouchers to the basket
 *
 * @param int $voucherActivityId Voucher activity ID
 * @return void
 */
	public function vouchers($voucherActivityId = null) {
		$GiftVoucherPurchase = $this->{$this->modelClass};
		$this->loadModel('BuzzVouchers.VoucherActivity');

		if (!empty($this->request->data)) {

			$result = $GiftVoucherPurchase->addItems($this->request->data['GiftVoucherPurchaseItem']);
			if ($result === true) {
				return $this->redirect(['action' => 'basket']);
			} else {
				$this->Session->setFlash(
					__d('BuzzVouchers', 'Please select a voucher'),
					'flash_fail'
				);
			}

		}

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		if (empty($voucherActivityId)) {
			// Check if the site is configured to only have one listed
			// voucher activity. This is needed to prevent a redirect
			// loop with this and the previous step.
			$voucherActivityId = $this->VoucherActivity->onlyAvailableActivity();
		}

		if ($voucherActivityId === false) {
			throw new NotFoundException();
		}

		// Get the vouchers and activity details for generating the form.
		$data = $this->VoucherActivity->getPurchasableVoucherActivity($voucherActivityId);

		if (empty($data)) {
			throw new NotFoundException();
		}

		if (empty($this->request->data['GiftVoucherPurchaseItem']) && !empty($basket['GiftVoucherPurchaseItem'])) {
			// Populate the form with existing basket items.
			$this->request->data['GiftVoucherPurchaseItem'] = Hash::combine(
				$basket['GiftVoucherPurchaseItem'],
				'{n}.gift_voucher_id',
				'{n}'
			);
		}

		$previousStep = ['action' => 'index'];

		$this->set(compact('data', 'basket', 'previousStep'));
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/vouchers';

		return;
	}

/**
 * Basket step - view the basket
 *
 * @return void
 */
	public function basket() {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		$previousStep = ['action' => 'index'];
		if (!empty($basket['GiftVoucherPurchaseItem'])) {
			$items = Hash::sort($basket['GiftVoucherPurchaseItem'], '{n}.modified', 'desc');
			$previousStep = array(
				'action' => 'vouchers',
				$items[0]['GiftVoucher']['voucher_activity_id']
			);
		}

		$this->set(compact('basket', 'previousStep'));
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/basket';

		return;
	}

/**
 * Checkout step
 *
 * @return void
 */
	public function checkout() {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		if (!empty($this->request->data)) {

		}

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		$previousStep = ['action' => 'basket'];

		$this->set(compact('basket', 'previousStep'));
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/checkout';

		return;
	}

}
