<?php

App::uses('BuzzVouchersAppModel', 'BuzzVouchers.Model');
App::uses('GiftVoucherPurchaseState', 'BuzzVouchers.Model');

class GiftVoucherPurchase extends BuzzVouchersAppModel {

/**
 * Belongs to associations
 *
 * @var array
 */
	public $belongsTo = array(
		'CustomerAddress' => array(
			'className' => 'BuzzCustomers.CustomerAddress'
		),
		'GiftVoucherPurchaseState' => array(
			'className' => 'BuzzVouchers.GiftVoucherPurchaseState'
		)
	);

/**
 * Has many associations
 *
 * @var array
 */
	public $hasMany = array(
		'GiftVoucherPurchaseItem' => array(
			'className' => 'BuzzVouchers.GiftVoucherPurchaseItem'
		)
	);

/**
 * Basket ID
 *
 * @var int
 */
	protected $_basketId = null;

/**
 * Get the current basket ID from the session
 *
 * @return int
 */
	public function getBasketId() {
		if (empty($this->_basketId)) {
			$this->_basketId = CakeSession::read('GiftVoucherPurchase.id');
		}
		return $this->_basketId;
	}

/**
 * Clear the basket ID from the session
 *
 * @return bool
 */
	public function clearBasketId() {
		$this->_basketId = null;
		CakeSession::delete('GiftVoucherPurchase');
		return true;
	}

/**
 * Get the current basket
 *
 * @return array
 */
	public function getBasket() {
		$basketId = $this->getBasketId();
		if (empty($basketId)) {
			return [];
		}
		$params = array(
			'contain' => array(
				'GiftVoucherPurchaseItem' => array(
					'GiftVoucher'
				)
			),
			'conditions' => array(
				'GiftVoucherPurchase.gift_voucher_purchase_state_id' => GiftVoucherPurchaseState::UNPAID
			)
		);
		return $this->readForView($basketId, $params);
	}

/**
 * Adds items to the basket
 *
 * @return bool
 */
	public function addItems($items) {
		$basketId = $this->getBasketId();

		// Make sure we don't save items with zero quantity to the database.
		$removeItems = [];
		foreach ($items as $key => $val) {
			if ((int)$val['quantity'] === 0) {
				unset($items[$key]);
			}
			// Flag existing items that are getting removed.
			if (!empty($val['id'])) {
				$removeItems[] = $val['id'];
			}
		}

		// Remove unwanted items.
		if (!empty($removeItems)) {
			$this->GiftVoucherPurchaseItem->deleteAll(['GiftVoucherPurchaseItem.id' => $removeItems]);
			unset($removeItems);
		}

		// Get the details of the gift vouchers being purchased so that we can
		// calculate costs.
		$vouchers = $this->GiftVoucherPurchaseItem->GiftVoucher->find(
			'all',
			array(
				'conditions' => array(
					'GiftVoucher.id' => Hash::extract($items, '{n}.gift_voucher_id')
				)
			)
		);
		$vouchers = Hash::combine($vouchers, '{n}.GiftVoucher.id', '{n}.GiftVoucher');

		foreach ($items as $key => &$item) {

			$giftVoucherId = $item['gift_voucher_id'];

			if (!empty($vouchers[$giftVoucherId])) {
				$item['item_unit_cost'] = $vouchers[$giftVoucherId]['price'];
				$item['total'] = $item['quantity'] * $vouchers[$giftVoucherId]['price'];
			} else {
				unset($items[$key]);
			}

		}

		if ($basketId === null) {
			// Create a new basket.
			$data = array(
				'GiftVoucherPurchase' => array(
					'id' => $basketId,
					'gift_voucher_purchase_state_id' => GiftVoucherPurchaseState::UNPAID
				),
				'GiftVoucherPurchaseItem' => $items
			);

			$result = $this->saveAssociated($data) !== false;
			$basketId = $result === true ? $this->id : null;

		} else {
			// Make sure we save to the current basket.
			$items = Hash::insert($items, '{n}.gift_voucher_purchase_id', $basketId);
			$result = $this->GiftVoucherPurchaseItem->saveMany($items) !== false;

		}

		// Update the session.
		if ($result === true) {
			CakeSession::write('GiftVoucherPurchase.id', $basketId);
		} else {
			$this->clearBasketId();
		}

		return $result;
	}

}
