<?php

class GiftVoucherPurchaseTest extends CakeTestCase {

	public $fixtures = array(
		'plugin.buzz_vouchers.GiftVoucherPurchase',
		'plugin.buzz_vouchers.GiftVoucherPurchaseItem',
		'plugin.buzz_vouchers.GiftVoucher',
	);

	public function setUp() {
		parent::setUp();
		$this->GiftVoucherPurchase = ClassRegistry::init('BuzzVouchers.GiftVoucherPurchase');
	}

/**
 * Make protected methods accessible for testing
 *
 * @param string $name Method name
 * @return object
 */
	public static function getMethod($name) {
		$GiftVoucherPurchase = new ReflectionClass('GiftVoucherPurchase');
		$method = $GiftVoucherPurchase->getMethod($name);
		$method->setAccessible(true);
		return $method;
	}

	public function testGetBasket() {
		// Test that we can retrieve a basket.
		$GiftVoucherPurchase = $this->getMockForModel('GiftVoucherPurchase', ['getBasketId']);
		$GiftVoucherPurchase->expects($this->any())->method('getBasketId')->will($this->returnValue(1));
		$result = $GiftVoucherPurchase->getBasket();
		$this->assertEquals(1, $result['GiftVoucherPurchase']['id']);
		return;
	}

	public function testGetBasketFails() {
		$GiftVoucherPurchase = $this->getMockForModel('GiftVoucherPurchase', ['getBasketId']);
		$GiftVoucherPurchase->expects($this->any())->method('getBasketId')->will($this->returnValue(2));
		$result = $GiftVoucherPurchase->getBasket();
		$this->assertEquals([], $result);
		return;
	}

/**
 * Test adding items to a new basket
 *
 * @return void
 */
	public function testAddItemsToNewBasket() {
		$GiftVoucherPurchase = $this->getMockForModel('GiftVoucherPurchase', ['getBasketId']);
		$GiftVoucherPurchase->expects($this->any())->method('getBasketId')->will($this->returnValue(null));
		$result = $GiftVoucherPurchase->addItems(
			array(
				array(
					'id' => null,
					'gift_voucher_id' => 1,
					'quantity' => 1
				),
				array(
					'id' => null,
					'gift_voucher_id' => 2,
					'quantity' => 2
				)
			)
		);
		$this->assertTrue($result);
		// Make sure the items have been created.
		$result = $GiftVoucherPurchase->GiftVoucherPurchaseItem->find(
			'all',
			array(
				'conditions' => array(
					'GiftVoucherPurchaseItem.gift_voucher_purchase_id' => $GiftVoucherPurchase->id
				)
			)
		);
		$this->assertCount(2, $result);
		// Check that the items are being summed correctly
		$this->assertEquals(30, $result[1]['GiftVoucherPurchaseItem']['total']);

		return;
	}

/**
 * Test removing items from an existing basket
 *
 * @return void
 */
	public function testAddItemsRemoveItem() {
		$GiftVoucherPurchase = $this->getMockForModel('GiftVoucherPurchase', ['getBasketId']);
		$GiftVoucherPurchase->expects($this->any())->method('getBasketId')->will($this->returnValue(1));
		$result = $GiftVoucherPurchase->addItems(
			array(
				array(
					'id' => 1,
					'gift_voucher_id' => 1,
					'quantity' => 1
				),
				array(
					'id' => 2,
					'gift_voucher_id' => 2,
					'quantity' => 0
				)
			)
		);
		$this->assertTrue($result);
		// Make sure the items have been updated.
		$result = $GiftVoucherPurchase->GiftVoucherPurchaseItem->find(
			'all',
			array(
				'conditions' => array(
					'GiftVoucherPurchaseItem.gift_voucher_purchase_id' => 1
				)
			)
		);
		$this->assertCount(1, $result);
		// Check that the items are being summed correctly
		$this->assertEquals(1, $result[0]['GiftVoucherPurchaseItem']['quantity']);

		return;
	}

}
